@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #include "config.h"
@@ -29,12 +29,11 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
-#define SN_API_NOT_YET_FROZEN
-#include <libsn/sn.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -43,9 +42,6 @@
 
 #include <dbus/dbus-glib.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-
 #include "bacon-video-widget.h"
 #include "totem-interface.h"
 #include "totem-statusbar.h"
@@ -56,7 +52,7 @@
 
 #include "totem-plugin-viewer-constants.h"
 #include "totem-plugin-viewer-options.h"
-#include "totempluginviewer-marshal.h"
+#include "marshal.h"
 
 GtkWidget *totem_statusbar_create (void);
 GtkWidget *totem_volume_create (void);
@@ -64,6 +60,7 @@ GtkWidget *totem_pp_create (void);
 
 #define VOLUME_DOWN_OFFSET (-0.08)
 #define VOLUME_UP_OFFSET (0.08)
+#define MINIMUM_VIDEO_SIZE 150
 
 /* For newer D-Bus version */
 #ifndef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
@@ -89,8 +86,9 @@ typedef enum {
 
 typedef GObjectClass TotemEmbeddedClass;
 
-typedef struct _TotemPlItem {
+typedef struct {
 	char *uri;
+	char *title;
 	int duration;
 	int starttime;
 } TotemPlItem;
@@ -122,7 +120,7 @@ typedef struct _TotemEmbedded {
 	int num_items;
 
 	/* Open menu item */
-	GnomeVFSMimeApplication *app;
+	GAppInfo *app;
 	GtkWidget *menu_item;
 
 	/* Seek bits */
@@ -629,237 +627,19 @@ totem_embedded_set_volume (TotemEmbedded *embedded,
 	return TRUE;
 }
 
-/* Copied from nautilus-program-choosing.c */
-
-extern char **environ;
-
-/* Cut and paste from gdkspawn-x11.c */
-static gchar **
-my_gdk_spawn_make_environment_for_screen (GdkScreen  *screen,
-					  gchar     **envp)
-{
-  gchar **retval = NULL;
-  gchar  *display_name;
-  gint    display_index = -1;
-  gint    i, env_len;
-
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-
-  if (envp == NULL)
-    envp = environ;
-
-  for (env_len = 0; envp[env_len]; env_len++)
-    if (strncmp (envp[env_len], "DISPLAY", strlen ("DISPLAY")) == 0)
-      display_index = env_len;
-
-  retval = g_new (char *, env_len + 1);
-  retval[env_len] = NULL;
-
-  display_name = gdk_screen_make_display_name (screen);
-
-  for (i = 0; i < env_len; i++)
-    if (i == display_index)
-      retval[i] = g_strconcat ("DISPLAY=", display_name, NULL);
-    else
-      retval[i] = g_strdup (envp[i]);
-
-  g_assert (i == env_len);
-
-  g_free (display_name);
-
-  return retval;
-}
-
-
-static void
-sn_error_trap_push (SnDisplay *display,
-		    Display   *xdisplay)
-{
-	gdk_error_trap_push ();
-}
-
-static void
-sn_error_trap_pop (SnDisplay *display,
-		   Display   *xdisplay)
-{
-	gdk_error_trap_pop ();
-}
-
-static char **
-make_spawn_environment_for_sn_context (SnLauncherContext *sn_context,
-				       char             **envp)
-{
-	char **retval;
-	int    i, j;
-
-	retval = NULL;
-	
-	if (envp == NULL) {
-		envp = environ;
-	}
-	
-	for (i = 0; envp[i]; i++) {
-		/* Count length */
-	}
-
-	retval = g_new (char *, i + 2);
-
-	for (i = 0, j = 0; envp[i]; i++) {
-		if (!g_str_has_prefix (envp[i], "DESKTOP_STARTUP_ID=")) {
-			retval[j] = g_strdup (envp[i]);
-			++j;
-	        }
-	}
-
-	retval[j] = g_strdup_printf ("DESKTOP_STARTUP_ID=%s",
-				     sn_launcher_context_get_startup_id (sn_context));
-	++j;
-	retval[j] = NULL;
-
-	return retval;
-}
-
-/* This should be fairly long, as it's confusing to users if a startup
- * ends when it shouldn't (it appears that the startup failed, and
- * they have to relaunch the app). Also the timeout only matters when
- * there are bugs and apps don't end their own startup sequence.
- *
- * This timeout is a "last resort" timeout that ignores whether the
- * startup sequence has shown activity or not.  Metacity and the
- * tasklist have smarter, and correspondingly able-to-be-shorter
- * timeouts. The reason our timeout is dumb is that we don't monitor
- * the sequence (don't use an SnMonitorContext)
- */
-#define STARTUP_TIMEOUT_LENGTH (30 /* seconds */ * 1000)
-
-typedef struct
-{
-	GdkScreen *screen;
-	GSList *contexts;
-	guint timeout_id;
-} StartupTimeoutData;
-
-static void
-free_startup_timeout (void *data)
-{
-	StartupTimeoutData *std;
-
-	std = data;
-
-	g_slist_foreach (std->contexts,
-			 (GFunc) sn_launcher_context_unref,
-			 NULL);
-	g_slist_free (std->contexts);
-
-	if (std->timeout_id != 0) {
-		g_source_remove (std->timeout_id);
-		std->timeout_id = 0;
-	}
-
-	g_free (std);
-}
-
-static gboolean
-startup_timeout (void *data)
-{
-	StartupTimeoutData *std;
-	GSList *tmp;
-	GTimeVal now;
-	int min_timeout;
-
-	std = data;
-
-	min_timeout = STARTUP_TIMEOUT_LENGTH;
-	
-	g_get_current_time (&now);
-	
-	tmp = std->contexts;
-	while (tmp != NULL) {
-		SnLauncherContext *sn_context;
-		GSList *next;
-		long tv_sec, tv_usec;
-		double elapsed;
-		
-		sn_context = tmp->data;
-		next = tmp->next;
-		
-		sn_launcher_context_get_last_active_time (sn_context,
-							  &tv_sec, &tv_usec);
-
-		elapsed =
-			((((double)now.tv_sec - tv_sec) * G_USEC_PER_SEC +
-			  (now.tv_usec - tv_usec))) / 1000.0;
-
-		if (elapsed >= STARTUP_TIMEOUT_LENGTH) {
-			std->contexts = g_slist_remove (std->contexts,
-							sn_context);
-			sn_launcher_context_complete (sn_context);
-			sn_launcher_context_unref (sn_context);
-		} else {
-			min_timeout = MIN (min_timeout, (STARTUP_TIMEOUT_LENGTH - elapsed));
-		}
-		
-		tmp = next;
-	}
-
-	if (std->contexts == NULL) {
-		std->timeout_id = 0;
-	} else {
-		std->timeout_id = g_timeout_add (min_timeout,
-						 startup_timeout,
-						 std);
-	}
-
-	/* always remove this one, but we may have reinstalled another one. */
-	return FALSE;
-}
-
-static void
-add_startup_timeout (GdkScreen         *screen,
-		     SnLauncherContext *sn_context)
-{
-	StartupTimeoutData *data;
-
-	data = g_object_get_data (G_OBJECT (screen), "nautilus-startup-data");
-	if (data == NULL) {
-		data = g_new (StartupTimeoutData, 1);
-		data->screen = screen;
-		data->contexts = NULL;
-		data->timeout_id = 0;
-		
-		g_object_set_data_full (G_OBJECT (screen),
-					"nautilus-startup-data",
-					data, free_startup_timeout);		
-	}
-
-	sn_launcher_context_ref (sn_context);
-	data->contexts = g_slist_prepend (data->contexts, sn_context);
-	
-	if (data->timeout_id == 0) {
-		data->timeout_id = g_timeout_add (STARTUP_TIMEOUT_LENGTH,
-						  startup_timeout,
-						  data);		
-	}
-}
-
 static gboolean
 totem_embedded_launch_player (TotemEmbedded *embedded,
-			      const char *uri,
- 			      guint32 user_time,
-			      GError *error)
+ 			      guint32 user_time)
 {
 	GList *uris = NULL;
 	GdkScreen *screen;
-	GnomeVFSResult result;
-	SnLauncherContext *sn_context;
-	SnDisplay *sn_display;
-	char **envp;
+	GAppLaunchContext *ctx;
+	gboolean result;
+	const char *uri;
 
 	g_return_val_if_fail (embedded->app != NULL, FALSE);
 
-	if (uri != NULL) {
-		uris = g_list_prepend (uris, (gpointer) uri);
-	} else if (embedded->type == TOTEM_PLUGIN_TYPE_NARROWSPACE
+	if (embedded->type == TOTEM_PLUGIN_TYPE_NARROWSPACE
 		   && embedded->href_uri != NULL) {
 		uris = g_list_prepend (uris, embedded->href_uri);
 		uri = embedded->href_uri;
@@ -868,75 +648,100 @@ totem_embedded_launch_player (TotemEmbedded *embedded,
 		uri = embedded->current_uri;
 	}
 
+	ctx = G_APP_LAUNCH_CONTEXT (gdk_app_launch_context_new ());
 	screen = gtk_widget_get_screen (embedded->window);
-	g_return_val_if_fail (screen != NULL, FALSE);
-	envp = my_gdk_spawn_make_environment_for_screen (screen, NULL);
+	gdk_app_launch_context_set_screen (GDK_APP_LAUNCH_CONTEXT (ctx), screen);
 
-	sn_display = sn_display_new (gdk_display,
-				     sn_error_trap_push,
-				     sn_error_trap_pop);
+	gdk_app_launch_context_set_timestamp (GDK_APP_LAUNCH_CONTEXT (ctx), user_time);
+	gdk_app_launch_context_set_icon (GDK_APP_LAUNCH_CONTEXT (ctx), g_app_info_get_icon (embedded->app));
 
-	if (gnome_vfs_mime_application_supports_startup_notification (embedded->app)) {
-		char *name;
-
-		sn_context = sn_launcher_context_new (sn_display,
-						      gdk_screen_get_number (screen));
-
-		name = g_filename_display_basename (uri);
-		if (name != NULL) {
-			char *description;
-
-			sn_launcher_context_set_name (sn_context, name);
-			description = g_strdup_printf (_("Opening %s"), name);
-			sn_launcher_context_set_description (sn_context,
-							     description);
-			g_free (name);
-			g_free (description);
-		}
-
-		if (!sn_launcher_context_get_initiated (sn_context)) {
-			const char *binary_name;
-			char **old_envp;
-
-			binary_name = gnome_vfs_mime_application_get_binary_name
-				(embedded->app);
-
-			sn_launcher_context_set_binary_name (sn_context,
-							     binary_name);
-
-			sn_launcher_context_initiate (sn_context,
-						      g_get_prgname (),
-						      binary_name,
-						      (Time) user_time);
-
-			old_envp = envp;
-			envp = make_spawn_environment_for_sn_context
-				(sn_context, envp);
-			g_strfreev (old_envp);
-		}
-	} else {
-		sn_context = NULL;
-  	}
-  
-	result = gnome_vfs_mime_application_launch_with_env (embedded->app,
-							     uris, envp);
-
-	if (sn_context != NULL) {
-		if (result != GNOME_VFS_OK) {
-			/* end sequence */
-			sn_launcher_context_complete (sn_context);
-		} else {
-			add_startup_timeout (screen, sn_context);
-		}
-		sn_launcher_context_unref (sn_context);
-	}
-
-	sn_display_unref (sn_display);
+	result = g_app_info_launch_uris (embedded->app, uris, ctx, NULL);
 
 	g_list_free (uris);
-	g_strfreev (envp);
 
-	return (result == GNOME_VFS_OK);
+	return result;
+}
+
+static char *
+relative_uri_remove_query (const char *uri, char **query)
+{
+	char *qmark;
+
+	/* Look for '?' */
+	qmark = strrchr (uri, '?');
+	if (qmark == NULL)
+		return NULL;
+
+	*query = g_strdup (qmark);
+	return g_strndup (uri, qmark - uri);
+}
+
+static char *
+resolve_uri (const char *base_uri,
+	     const char *relative_uri)
+{
+	char *uri, *scheme, *query, *new_relative_uri;
+	GFile *base_gfile, *base_parent_gfile, *resolved_gfile;
+
+	if (!relative_uri)
+		return g_strdup (base_uri);
+
+	if (!base_uri)
+		return g_strdup (relative_uri);
+
+	/* If |relative_uri| has a scheme, it's a full URI, just return it */
+	scheme = g_uri_parse_scheme (relative_uri);
+	if (scheme) {
+		g_free (scheme);
+		return g_strdup (relative_uri);
+	}
+
+	base_gfile = g_file_new_for_uri (base_uri);
+	base_parent_gfile = g_file_get_parent (base_gfile);
+	if (!base_parent_gfile) {
+		g_print ("Base URI %s has no parent!\n", base_uri);
+		g_object_unref (base_gfile);
+		return NULL;
+	}
+	g_object_unref (base_gfile);
+
+	query = NULL;
+	new_relative_uri = relative_uri_remove_query (relative_uri, &query);
+
+	if (new_relative_uri) {
+		char *tmpuri;
+
+		resolved_gfile = g_file_resolve_relative_path (base_parent_gfile, new_relative_uri);
+		g_object_unref (base_parent_gfile);
+		if (!resolved_gfile) {
+			g_warning ("Failed to resolve relative URI '%s' against base '%s'\n", relative_uri, base_uri);
+			g_free (new_relative_uri);
+			g_free (query);
+			return NULL;
+		}
+
+		tmpuri = g_file_get_uri (resolved_gfile);
+		g_object_unref (resolved_gfile);
+		uri = g_strdup_printf ("%s%s", tmpuri, query);
+
+		g_free (tmpuri);
+		g_free (new_relative_uri);
+		g_free (query);
+
+		return uri;
+	} else {
+		resolved_gfile = g_file_resolve_relative_path (base_parent_gfile, relative_uri);
+		g_object_unref (base_parent_gfile);
+		if (!resolved_gfile) {
+			g_warning ("Failed to resolve relative URI '%s' against base '%s'\n", relative_uri, base_uri);
+			return NULL;
+		}
+
+		uri = g_file_get_uri (resolved_gfile);
+		g_object_unref (resolved_gfile);
+
+		return uri;
+	}
 }
 
 static void
@@ -951,11 +756,13 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	old_base = emb->base_uri;
 	old_href = emb->href_uri;
 
-	emb->current_uri = g_strdup (uri);
 	emb->base_uri = g_strdup (base_uri);
+	emb->current_uri = resolve_uri (base_uri, uri);
 	emb->is_browser_stream = (is_browser_stream != FALSE);
 	emb->href_uri = NULL;
 
+        g_print ("totem_embedded_set_uri uri %s base %s => resolved %s\n", uri, base_uri, emb->current_uri);
+
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
@@ -963,6 +770,16 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	emb->stream_uri = NULL;
 }
 
+static void
+totem_embedded_update_title (TotemEmbedded *emb, const char *title)
+{
+	if (title == NULL)
+		gtk_window_set_title (GTK_WINDOW (emb->fs_window), _("Totem Movie Player"));
+	else
+		gtk_window_set_title (GTK_WINDOW (emb->fs_window), title);
+	totem_fullscreen_set_title (emb->fs, title);
+}
+
 static void
 totem_pl_item_free (gpointer data, gpointer user_data)
 {
@@ -971,22 +788,24 @@ totem_pl_item_free (gpointer data, gpointer user_data)
 	if (!item)
 		return;
 	g_free (item->uri);
+	g_free (item->title);
 	g_free (item);
 }
 
 static gboolean
-totem_embedded_clear_playlist (TotemEmbedded *embedded, GError *error)
+totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 {
-	g_list_foreach (embedded->playlist, (GFunc) totem_pl_item_free, NULL);
-	g_list_free (embedded->playlist);
+	g_list_foreach (emb->playlist, (GFunc) totem_pl_item_free, NULL);
+	g_list_free (emb->playlist);
 
-	embedded->playlist = NULL;
-	embedded->current = NULL;
-	embedded->num_items = 0;
+	emb->playlist = NULL;
+	emb->current = NULL;
+	emb->num_items = 0;
 
-	totem_embedded_set_uri (embedded, NULL, NULL, FALSE);
+	totem_embedded_set_uri (emb, NULL, NULL, FALSE);
 
-	bacon_video_widget_close (embedded->bvw);
+	bacon_video_widget_close (emb->bvw);
+	totem_embedded_update_title (emb, NULL);
 
 	return TRUE;
 }
@@ -1150,6 +969,7 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 			        FALSE);
 
 	bacon_video_widget_close (emb->bvw);
+	totem_embedded_update_title (emb, plitem->title);
 	if (totem_embedded_open_internal (emb, FALSE, NULL /* FIXME */)) {
 		if (plitem->starttime > 0) {
 			gboolean retval;
@@ -1244,6 +1064,19 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	return TRUE;
 }
 
+static GAppInfo *
+totem_embedded_get_app_for_uri (const char *uri)
+{
+	char *type;
+	GAppInfo *info;
+
+	type = g_content_type_guess (uri, NULL, 0, NULL);
+	info = g_app_info_get_default_for_type (type, TRUE);
+	g_free (type);
+
+	return info;
+}
+
 static void
 totem_embedded_update_menu (TotemEmbedded *emb)
 {
@@ -1256,13 +1089,12 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 		emb->menu_item = NULL;
 	}
 	if (emb->app != NULL) {
-		gnome_vfs_mime_application_free (emb->app);
+		g_object_unref (emb->app);
 		emb->app = NULL;
 	}
 
 	if (emb->mimetype && strcmp (emb->mimetype, "application/octet-stream") != 0) {
-		emb->app = gnome_vfs_mime_get_default_application_for_uri
-				(emb->current_uri, emb->mimetype);
+		emb->app = g_app_info_get_default_for_type (emb->mimetype, TRUE);
 	} else {
 		const char *uri;
 
@@ -1272,17 +1104,19 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 			uri = emb->stream_uri;
 		else
 			uri = emb->current_uri;
-		emb->app = gnome_vfs_mime_get_default_application_for_uri
-				(uri, gnome_vfs_get_mime_type_for_name (uri));
+		emb->app = totem_embedded_get_app_for_uri (uri);
 	}
 
 	if (emb->app == NULL) {
 		if (emb->mimetype != NULL) {
 			g_message ("Mimetype '%s' doesn't have a handler", emb->mimetype);
 		} else {
+			char *type;
+
+			type = g_content_type_guess (emb->current_uri, NULL, 0, NULL);
 			g_message ("No handler for URI '%s' (guessed mime-type '%s')",
-				   emb->current_uri,
-				   gnome_vfs_get_mime_type_for_name (emb->current_uri));
+				   emb->current_uri, type);
+			g_free (type);
 		}
 		return;
 	}
@@ -1290,7 +1124,7 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 	/* translators: this is:
 	 * Open With ApplicationName
 	 * as in nautilus' right-click menu */
-	label = g_strdup_printf (_("_Open with \"%s\""), emb->app->name);
+	label = g_strdup_printf (_("_Open with \"%s\""), g_app_info_get_name (emb->app));
 	item = gtk_image_menu_item_new_with_mnemonic (label);
 	g_free (label);
 	image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
@@ -1301,7 +1135,7 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 	emb->menu_item = item;
 
 	menu = GTK_MENU_SHELL (gtk_builder_get_object (emb->menuxml, "menu"));
-	gtk_menu_shell_prepend (menu, item);
+	gtk_menu_shell_insert (menu, item, 1);
 }
 
 static void
@@ -1309,7 +1143,7 @@ on_open1_activate (GtkButton *button, TotemEmbedded *emb)
 {
 	GTimeVal val;
 	g_get_current_time (&val);
-	totem_embedded_launch_player (emb, NULL, val.tv_sec, NULL);
+	totem_embedded_launch_player (emb, val.tv_sec);
 	totem_embedded_stop (emb, NULL);
 }
 
@@ -1346,9 +1180,10 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	license = totem_interface_get_license ();
 
 	emb->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
-				   "name", _("Totem Browser Plugin"),
+				   "program-name", _("Totem Browser Plugin"),
 				   "version", VERSION,
-				   "copyright", _("Copyright \xc2\xa9 2002-2007 Bastien Nocera"),
+				   "copyright", "Copyright © 2002-2007 Bastien Nocera\n"
+                                                "Copyright © 2006, 2007, 2008 Christian Persch",
 				   "comments", description,
 				   "authors", authors,
 				   "translator-credits", _("translator-credits"),
@@ -1498,10 +1333,31 @@ on_popup_button_button_pressed (GtkToggleButton *button,
 	return FALSE;
 }
 
+static char *
+resolve_redirect (const char *old_mrl, const char *mrl)
+{
+	GFile *old_file, *parent, *new_file;
+	char *retval;
+
+	/* Get the parent for the current MRL, that's our base */
+	old_file = g_file_new_for_uri (old_mrl);
+	parent = g_file_get_parent (old_file);
+	g_object_unref (old_file);
+
+	/* Resolve the URL */
+	new_file = g_file_get_child (parent, mrl);
+	g_object_unref (parent);
+
+	retval = g_file_get_uri (new_file);
+	g_object_unref (new_file);
+
+	return retval;
+}
+
 static void
 on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 {
-	char *new_uri = NULL;
+	char *new_uri;
 
 	g_message ("stream uri: %s", emb->stream_uri ? emb->stream_uri : "(null)");
 	g_message ("current uri: %s", emb->current_uri ? emb->current_uri : "(null)");
@@ -1510,29 +1366,81 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	bacon_video_widget_close (emb->bvw);
 
+	/* If we don't have a relative URI */
+	if (strstr (mrl, "://") != NULL)
+		new_uri = NULL;
 	/* We are using a local cache, so we resolve against the stream_uri */
 	if (emb->stream_uri)
-		new_uri = totem_pl_parser_resolve_url (emb->stream_uri, mrl);
+		new_uri = resolve_redirect (emb->stream_uri, mrl);
 	/* We don't have a local cache, so resolve against the URI */
 	else if (emb->current_uri)
-		new_uri = totem_pl_parser_resolve_url (emb->current_uri, mrl);
+		new_uri = resolve_redirect (emb->current_uri, mrl);
 	/* FIXME: not sure that this is actually correct... */
 	else if (emb->base_uri)
-		new_uri = totem_pl_parser_resolve_url (emb->base_uri, mrl);
+		new_uri = resolve_redirect (emb->base_uri, mrl);
 
-	if (!new_uri)
-		return;
-
-	g_message ("Redirecting to '%s'", new_uri);
+	g_message ("Redirecting to '%s'", new_uri ? new_uri : mrl);
 
 	/* FIXME: clear playlist? or replace current entry? or add a new entry? */
 	/* FIXME: use totem_embedded_open_uri? */
 
-	totem_embedded_set_uri (emb, new_uri, emb->base_uri /* FIXME? */, FALSE);
+	totem_embedded_set_uri (emb, new_uri ? new_uri : mrl , emb->base_uri /* FIXME? */, FALSE);
 
 	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
 	totem_embedded_open_internal (emb, TRUE, NULL /* FIXME? */);
+
+	g_free (new_uri);
+}
+
+static char *
+totem_embedded_get_nice_name_for_stream (BaconVideoWidget *bvw)
+{
+	char *title, *artist, *retval;
+	int tracknum;
+	GValue value = { 0, };
+
+	bacon_video_widget_get_metadata (bvw, BVW_INFO_TITLE, &value);
+	title = g_value_dup_string (&value);
+	g_value_unset (&value);
+
+	if (title == NULL)
+		return NULL;
+
+	bacon_video_widget_get_metadata (bvw, BVW_INFO_ARTIST, &value);
+	artist = g_value_dup_string (&value);
+	g_value_unset (&value);
+
+	if (artist == NULL)
+		return title;
+
+	bacon_video_widget_get_metadata (bvw,
+					 BVW_INFO_TRACK_NUMBER,
+					 &value);
+	tracknum = g_value_get_int (&value);
+
+	if (tracknum != 0) {
+		retval = g_strdup_printf ("%02d. %s - %s",
+				tracknum, artist, title);
+	} else {
+		retval = g_strdup_printf ("%s - %s", artist, title);
+	}
+	g_free (artist);
+	g_free (title);
+
+	return retval;
+}
+
+static void
+on_got_metadata (BaconVideoWidget *bvw, TotemEmbedded *emb)
+{
+	char *title;
+
+	title = totem_embedded_get_nice_name_for_stream (bvw);
+	if (title == NULL)
+		return;
+
+	totem_embedded_update_title (emb, title);
 }
 
 static void
@@ -1585,6 +1493,8 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 
 		gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
 				event->button, event->time);
+	} else if (event->type == GDK_2BUTTON_PRESS && event->button == 1 && state == 0) {
+		totem_embedded_toggle_fullscreen (emb);
 	}
 
 	return FALSE;
@@ -1777,6 +1687,15 @@ controls_size_allocate_cb (GtkWidget *controls,
 }
 #endif
 
+static void
+video_widget_size_allocate_cb (GtkWidget *controls,
+			       GtkAllocation *allocation,
+			       BaconVideoWidget *bvw)
+{
+	bacon_video_widget_set_show_visuals (bvw, allocation->height > MINIMUM_VIDEO_SIZE);
+	g_signal_handlers_disconnect_by_func (controls, G_CALLBACK (video_widget_size_allocate_cb), NULL);
+}
+
 static void
 totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
 {
@@ -1895,8 +1814,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	}
 
 	/* FIXME! */
-	if (emb->bvw == NULL)
-	{
+	if (emb->bvw == NULL) {
 		/* FIXME! */
 		/* FIXME construct and show error message */
 		totem_embedded_error_and_exit (_("The Totem plugin could not be started."), err != NULL ? err->message : _("No reason."), emb);
@@ -1938,6 +1856,8 @@ totem_embedded_construct (TotemEmbedded *emb,
 
 	g_signal_connect (G_OBJECT(emb->bvw), "got-redirect",
 			G_CALLBACK (on_got_redirect), emb);
+	g_signal_connect (G_OBJECT(emb->bvw), "got-metadata",
+			  G_CALLBACK (on_got_metadata), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "eos",
 			G_CALLBACK (on_eos_event), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "error",
@@ -1958,7 +1878,10 @@ totem_embedded_construct (TotemEmbedded *emb,
 		/* FIXME: why can't this wait until the whole window is realised? */
 		gtk_widget_realize (GTK_WIDGET (emb->bvw));
 		gtk_widget_show (GTK_WIDGET (emb->bvw));
-		bacon_video_widget_set_show_visuals (emb->bvw, TRUE);
+
+		/* Let us know when the size has been allocated for the video widget */
+		g_signal_connect_after (G_OBJECT (emb->bvw), "size-allocate",
+					G_CALLBACK (video_widget_size_allocate_cb), emb->bvw);
 	} else if (emb->audioonly != FALSE) {
 		gtk_widget_hide (container);
 	}
@@ -2189,14 +2112,15 @@ entry_parsed (TotemPlParser *parser,
 	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 
 	/* Skip short advert streams */
-	duration = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "duration"), FALSE);
+	duration = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DURATION), FALSE);
 	if (duration == 0)
 		return;
 
 	item = g_new0 (TotemPlItem, 1);
 	item->uri = g_strdup (uri);
+	item->title = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE));
 	item->duration = duration;
-	item->starttime = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "starttime"), FALSE);
+	item->starttime = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_STARTTIME), FALSE);
 
 	emb->playlist = g_list_prepend (emb->playlist, item);
 }
@@ -2402,6 +2326,8 @@ int main (int argc, char **argv)
 
 	/* FIXME: check the UA strings of the legacy plugins themselves */
 	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
+	/* FIXME: this won't work with gvfs:
+	 * http://bugzilla.gnome.org/show_bug.cgi?id=534482 */
 	if (arg_user_agent != NULL) {
 		g_setenv ("GNOME_VFS_HTTP_USER_AGENT", arg_user_agent, TRUE);
 		g_free (arg_user_agent);
@@ -2409,7 +2335,6 @@ int main (int argc, char **argv)
 	}
 
 	bacon_video_widget_init_backend (NULL, NULL);
-	gnome_vfs_init ();
 
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
 					 &dbus_glib_totem_embedded_object_info);
