@@ -35,10 +35,10 @@
 G_BEGIN_DECLS
 
 #define TOTEM_TYPE_OPEN_LOCATION		(totem_open_location_get_type ())
-#define TOTEM_OPEN_LOCATION(obj)		(GTK_CHECK_CAST ((obj), TOTEM_TYPE_OPEN_LOCATION, TotemOpenLocation))
-#define TOTEM_OPEN_LOCATION_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), TOTEM_TYPE_OPEN_LOCATION, TotemOpenLocationClass))
-#define TOTEM_IS_OPEN_LOCATION(obj)		(GTK_CHECK_TYPE ((obj), TOTEM_TYPE_OPEN_LOCATION))
-#define TOTEM_IS_OPEN_LOCATION_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), TOTEM_TYPE_OPEN_LOCATION))
+#define TOTEM_OPEN_LOCATION(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), TOTEM_TYPE_OPEN_LOCATION, TotemOpenLocation))
+#define TOTEM_OPEN_LOCATION_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), TOTEM_TYPE_OPEN_LOCATION, TotemOpenLocationClass))
+#define TOTEM_IS_OPEN_LOCATION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), TOTEM_TYPE_OPEN_LOCATION))
+#define TOTEM_IS_OPEN_LOCATION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), TOTEM_TYPE_OPEN_LOCATION))
 
 typedef struct TotemOpenLocation		TotemOpenLocation;
 typedef struct TotemOpenLocationClass		TotemOpenLocationClass;
@@ -53,7 +53,7 @@ struct TotemOpenLocationClass {
 	GtkDialogClass parent_class;
 };
 
-GtkType totem_open_location_get_type		(void);
+GType totem_open_location_get_type		(void);
 GtkWidget *totem_open_location_new		(Totem *totem);
 char *totem_open_location_get_uri		(TotemOpenLocation *open_location);
 
