@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -31,7 +31,9 @@ void totem_sidebar_toggle (Totem *totem, gboolean state);
 void totem_sidebar_set_visibility (Totem *totem, gboolean visible);
 gboolean totem_sidebar_is_visible (Totem *totem);
 char *totem_sidebar_get_current_page (Totem *totem);
-void totem_sidebar_set_current_page (Totem *totem, const char *name);
+void totem_sidebar_set_current_page (Totem *totem,
+				     const char *name,
+				     gboolean force_visible);
 
 G_END_DECLS
 
