@@ -52,6 +52,7 @@
 #endif
 
 #include "bacon-video-widget.h"
+#include "totem-dvb-setup.h"
 #include "totem-statusbar.h"
 #include "totem-time-label.h"
 #include "totem-session.h"
@@ -97,6 +98,7 @@ static const GtkTargetEntry target_table[] = {
 
 static gboolean totem_action_open_files (Totem *totem, char **list);
 static gboolean totem_action_open_files_list (Totem *totem, GSList *list);
+static gboolean totem_action_load_media (Totem *totem, TotemDiscMediaType type, const char *device);
 static void update_buttons (Totem *totem);
 static void update_media_menu_items (Totem *totem);
 static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
@@ -516,28 +518,33 @@ totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 	return TRUE;
 }
 
+static void
+totem_dvb_setup_result (int result, const char *device, gpointer user_data)
+{
+	Totem *totem = (Totem *) user_data;
+
+	totem_action_play_media (totem, MEDIA_TYPE_DVB, device);
+}
+
 static gboolean
 totem_action_load_media (Totem *totem, TotemDiscMediaType type, const char *device)
 {
-	char **mrls;
-	char *msg;
+	char **mrls, *msg;
+	GError *error = NULL;
+	const char *link, *link_text, *secondary;
 	gboolean retval;
-	BaconVideoWidgetCanPlayStatus status;
 
-	status = bacon_video_widget_can_play (totem->bvw, type);
-
-	if (status != BVW_CAN_PLAY_SUCCESS) {
-		const char *link, *link_text, *secondary;
-
-		switch (status) {
-		case BVW_CAN_PLAY_MISSING_CHANNELS:
-			g_assert (type == MEDIA_TYPE_DVB);
-			link = "http://www.gnome.org/projects/totem/#dvb";
-			link_text = _("More information about watching TV");
-			msg = g_strdup (_("Totem is missing a channels listing to be able to tune the receiver."));
-			secondary = _("Please follow the instructions provided in the link to create a channels listing.");
-			break;
-		case BVW_CAN_PLAY_MISSING_PLUGINS:
+	mrls = bacon_video_widget_get_mrls (totem->bvw, type, device, &error);
+	if (mrls == NULL) {
+		/* No errors? Weird */
+		if (error == NULL) {
+			msg = g_strdup_printf (_("Totem could not play this media (%s) although a plugin is present to handle it."), _(totem_cd_get_human_readable_name (type)));
+			totem_action_error (msg, _("You might want to check that a disc is present in the drive and that it is correctly configured."), totem);
+			g_free (msg);
+			return FALSE;
+		}
+		/* No plugin for the media type */
+		if (g_error_matches (error, BVW_ERROR, BVW_ERROR_NO_PLUGIN_FOR_FILE) != FALSE) {
 			link = "http://www.gnome.org/projects/totem/#codecs";
 			link_text = _("More information about media plugins");
 			secondary = _("Please install the necessary plugins and restart Totem to be able to play this media.");
@@ -545,29 +552,43 @@ totem_action_load_media (Totem *totem, TotemDiscMediaType type, const char *devi
 				msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because it does not have the appropriate plugins to be able to read from the disc."), _(totem_cd_get_human_readable_name (type)));
 			else
 				msg = g_strdup_printf (_("Totem cannot play this type of media (%s) because you do not have the appropriate plugins to handle it."), _(totem_cd_get_human_readable_name (type)));
-			break;
-		case BVW_CAN_PLAY_UNSUPPORTED:
+		/* Device doesn't exist */
+		} else if (g_error_matches (error, BVW_ERROR, BVW_ERROR_INVALID_DEVICE) != FALSE) {
+			g_assert (type == MEDIA_TYPE_DVB);
+			msg = N_("Totem cannot play TV, because no TV adapters are present or they are not supported.");
+			totem_action_error (_(msg), _("Please insert a supported TV adapter."), totem);
+			return FALSE;
+		/* No channels.conf file */
+		} else if (g_error_matches (error, BVW_ERROR, BVW_ERROR_FILE_NOT_FOUND) != FALSE) {
+			g_assert (type == MEDIA_TYPE_DVB);
+
+			if (totem_dvb_setup_device (device, GTK_WINDOW (totem->win), totem_dvb_setup_result, totem) == TOTEM_DVB_SETUP_STARTED_OK)
+				return FALSE;
+
+			link = "http://www.gnome.org/projects/totem/#dvb";
+			link_text = _("More information about watching TV");
+			msg = g_strdup (_("Totem is missing a channels listing to be able to tune the receiver."));
+			secondary = _("Please follow the instructions provided in the link to create a channels listing.");
+		} else if (g_error_matches (error, BVW_ERROR, BVW_ERROR_DEVICE_BUSY) != FALSE) {
+			g_assert (type == MEDIA_TYPE_DVB);
+			msg = g_strdup_printf(_("Totem cannot play this type of media (%s) because the TV device is busy."), _(totem_cd_get_human_readable_name (type)));
+			totem_action_error (msg, _("Please try again later."), totem);
+			g_free (msg);
+			return FALSE;
+		/* Unsupported type (ie. CDDA) */
+		} else if (g_error_matches (error, BVW_ERROR, BVW_ERROR_UNVALID_LOCATION) != FALSE) {
 			msg = g_strdup_printf(_("Totem cannot play this type of media (%s) because it is not supported."), _(totem_cd_get_human_readable_name (type)));
 			totem_action_error (msg, _("Please insert another disc to play back."), totem);
 			g_free (msg);
 			return FALSE;
-		default:
+		} else {
 			g_assert_not_reached ();
 		}
-
 		totem_interface_error_with_link (msg, secondary, link, link_text, GTK_WINDOW (totem->win), totem);
 		g_free (msg);
 		return FALSE;
 	}
 
-	mrls = bacon_video_widget_get_mrls (totem->bvw, type, device);
-	if (mrls == NULL) {
-		msg = g_strdup_printf (_("Totem could not play this media (%s) although a plugin is present to handle it."), _(totem_cd_get_human_readable_name (type)));
-		totem_action_error (msg, _("You might want to check that a disc is present in the drive and that it is correctly configured."), totem);
-		g_free (msg);
-		return FALSE;
-	}
-
 	retval = totem_action_open_files (totem, mrls);
 	g_strfreev (mrls);
 
@@ -955,6 +976,7 @@ totem_get_nice_name_for_stream (Totem *totem)
 					 BVW_INFO_TRACK_NUMBER,
 					 &value);
 	tracknum = g_value_get_int (&value);
+	g_value_unset (&value);
 
 	if (tracknum != 0) {
 		retval = g_strdup_printf ("%02d. %s - %s",
@@ -971,17 +993,8 @@ totem_get_nice_name_for_stream (Totem *totem)
 static void
 update_mrl_label (Totem *totem, const char *name)
 {
-	gint time;
-
 	if (name != NULL)
 	{
-		/* Get the length of the stream */
-		time = bacon_video_widget_get_stream_length (totem->bvw);
-		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
-						     (totem->statusbar), 0, time / 1000);
-
-		g_object_notify (G_OBJECT (totem), "stream-length");
-
 		/* Update the mrl label */
 		totem_fullscreen_set_title (totem->fs, name);
 
@@ -1035,14 +1048,14 @@ totem_action_set_mrl_with_warning (Totem *totem,
 		totem->mrl = NULL;
 		bacon_video_widget_close (totem->bvw);
 		totem_file_closed (totem);
-		play_pause_set_label (totem, TOTEM_PLAYLIST_STATUS_NONE);
+		play_pause_set_label (totem, STATE_STOPPED);
 	}
 
 	if (mrl == NULL)
 	{
 		retval = FALSE;
 
-		play_pause_set_label (totem, TOTEM_PLAYLIST_STATUS_NONE);
+		play_pause_set_label (totem, STATE_STOPPED);
 
 		/* Play/Pause */
 		totem_action_set_sensitivity ("play", FALSE);
@@ -1343,7 +1356,7 @@ totem_action_zoom_relative (Totem *totem, int off_pct)
 void
 totem_action_zoom_reset (Totem *totem)
 {
-	totem_action_zoom (totem, 100);
+	totem_action_zoom (totem, ZOOM_RESET);
 }
 
 /**
@@ -1432,15 +1445,12 @@ totem_action_set_scale_ratio (Totem *totem, gfloat ratio)
 void
 totem_action_show_help (Totem *totem)
 {
-#ifndef HAVE_GTK_ONLY
-	GError *err = NULL;
+	GError *error = NULL;
 
-	if (gnome_help_display ("totem.xml", NULL, &err) == FALSE)
-	{
-		totem_action_error (_("Totem could not display the help contents."), err->message, totem);
-		g_error_free (err);
+	if (gtk_show_uri (gtk_widget_get_screen (totem->win), "ghelp:totem", gtk_get_current_event_time (), &error) == FALSE) {
+		totem_action_error (_("Totem could not display the help contents."), error->message, totem);
+		g_error_free (error);
 	}
-#endif /* !HAVE_GTK_ONLY */
 }
 
 static gboolean
@@ -1699,6 +1709,10 @@ static void
 on_error_event (BaconVideoWidget *bvw, char *message,
                 gboolean playback_stopped, gboolean fatal, Totem *totem)
 {
+	/* Clear the seek if it's there, we only want to try and seek
+	 * the first file, even if it's not there */
+	totem->seek_to = 0;
+
 	if (playback_stopped)
 		play_pause_set_label (totem, STATE_STOPPED);
 
@@ -1747,6 +1761,14 @@ update_seekable (Totem *totem)
 	action = gtk_action_group_get_action (action_group, "skip-backwards");
 	gtk_action_set_sensitive (action, seekable);
 
+	/* This is for the session restore to seek */
+	if (seekable != FALSE && totem->seek_to != 0) {
+		bacon_video_widget_seek_time (totem->bvw,
+				totem->seek_to, NULL);
+		totem_action_pause (totem);
+	}
+	totem->seek_to = 0;
+
 	g_object_notify (G_OBJECT (totem), "seekable");
 }
 
@@ -1833,9 +1855,18 @@ property_notify_cb_seekable (BaconVideoWidget *bvw, GParamSpec *spec, Totem *tot
 gboolean
 seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 {
+	/* HACK: we want the behaviour you get with the middle button, so we
+	 * mangle the event.  clicking with other buttons moves the slider in
+	 * step increments, clicking with the middle button moves the slider to
+	 * the location of the click.
+	 */
+	event->button = 2;
+
 	totem->seek_lock = TRUE;
-	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
-	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), TRUE);
+	if (bacon_video_widget_can_direct_seek (totem->bvw) == FALSE) {
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), TRUE);
+	}
 
 	return FALSE;
 }
@@ -1867,6 +1898,9 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	GtkAdjustment *adj;
 	gdouble val;
 
+	/* HACK: see seek_slider_pressed_cb */
+	event->button = 2;
+
 	/* set to FALSE here to avoid triggering a final seek when
 	 * syncing the adjustments while being in direct seek mode */
 	totem->seek_lock = FALSE;
@@ -1960,7 +1994,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
 				changed = TRUE;
 			} else if (g_str_equal (filename, "dvb:") != FALSE) {
-				changed = totem_action_load_media (totem, MEDIA_TYPE_DVB, NULL);
+				totem_action_load_media (totem, MEDIA_TYPE_DVB, "0");
+				changed = TRUE;
 			} else if (totem_playlist_add_mrl (totem->playlist, filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
 				changed = TRUE;
@@ -2027,6 +2062,7 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 			gtk_widget_style_get_property (pane, "handle-size",
 					&value);
 			handle_size = g_value_get_int (&value);
+			g_value_unset (&value);
 			
 			gtk_widget_show (totem->sidebar);
 			width += totem->sidebar->allocation.width
@@ -2140,12 +2176,44 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_PAUSE:
 		totem_action_pause (totem);
 		break;
-	case TOTEM_REMOTE_COMMAND_SEEK_FORWARD:
-		totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000);
+	case TOTEM_REMOTE_COMMAND_STOP: {
+		char *mrl, *subtitle;
+
+		totem_playlist_set_at_start (totem->playlist);
+		update_buttons (totem);
+		totem_action_stop (totem);
+		mrl = totem_playlist_get_current_mrl (totem->playlist, &subtitle);
+		if (mrl != NULL) {
+			totem_action_set_mrl_with_warning (totem, mrl, subtitle, FALSE);
+			bacon_video_widget_pause (totem->bvw);
+			g_free (mrl);
+			g_free (subtitle);
+		}
 		break;
-	case TOTEM_REMOTE_COMMAND_SEEK_BACKWARD:
-		totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000);
+	};
+	case TOTEM_REMOTE_COMMAND_SEEK_FORWARD: {
+		double offset = 0;
+
+		if (url != NULL)
+			offset = g_ascii_strtod (url, NULL);
+		if (offset == 0) {
+			totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000);
+		} else {
+			totem_action_seek_relative (totem, offset * 1000);
+		}
 		break;
+	}
+	case TOTEM_REMOTE_COMMAND_SEEK_BACKWARD: {
+		double offset = 0;
+
+		if (url != NULL)
+			offset = g_ascii_strtod (url, NULL);
+		if (offset == 0)
+			totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000);
+		else
+			totem_action_seek_relative (totem,  - (offset * 1000));
+		break;
+	}
 	case TOTEM_REMOTE_COMMAND_VOLUME_UP:
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
 		break;
@@ -2185,7 +2253,7 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 			/* FIXME b0rked */
 			totem_action_play_media (totem, MEDIA_TYPE_VCD, NULL);
 		} else if (g_str_has_prefix (url, "dvb:") != FALSE) {
-			totem_action_play_media (totem, MEDIA_TYPE_DVB, NULL);
+			totem_action_load_media (totem, MEDIA_TYPE_DVB, "0");
 		} else if (totem_playlist_add_mrl_with_cursor (totem->playlist, url, NULL) != FALSE) {
 			totem_action_add_recent (totem, url);
 		}
@@ -2515,6 +2583,15 @@ on_eos_event (GtkWidget *widget, Totem *totem)
 	if (bacon_video_widget_get_logo_mode (totem->bvw) != FALSE)
 		return FALSE;
 
+	/* EOS on DVB means that we lost the signal */
+	if (totem->mrl != NULL && g_str_has_prefix (totem->mrl, "dvb://") != FALSE) {
+		totem_action_stop (totem);
+		totem_action_error_and_exit (_("TV signal lost"),
+					     _("Please verify your hardware setup."),
+					     totem);
+		return FALSE;
+	}
+
 	if (totem_playlist_has_next_mrl (totem->playlist) == FALSE
 			&& totem_playlist_get_repeat (totem->playlist) == FALSE)
 	{
@@ -2755,17 +2832,19 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		break;
 	case GDK_plus:
 	case GDK_KP_Add:
-		if (!(event->state & GDK_CONTROL_MASK))
-			return FALSE;
-
-		totem_action_zoom_relative (totem, ZOOM_IN_OFFSET);
+		if (!(event->state & GDK_CONTROL_MASK)) {
+			totem_action_next (totem);
+		} else {
+			totem_action_zoom_relative (totem, ZOOM_IN_OFFSET);
+		}
 		break;
 	case GDK_minus:
 	case GDK_KP_Subtract:
-		if (!(event->state & GDK_CONTROL_MASK))
-			return FALSE;
-
-		totem_action_zoom_relative (totem, ZOOM_OUT_OFFSET);
+		if (!(event->state & GDK_CONTROL_MASK)) {
+			totem_action_previous (totem);
+		} else {
+			totem_action_zoom_relative (totem, ZOOM_OUT_OFFSET);
+		}
 		break;
 	case GDK_KP_Up:
 	case GDK_KP_8:
@@ -3057,6 +3136,8 @@ totem_callback_connect (Totem *totem)
 	/* Menu items */
 	gtk_action_group_set_visible (totem->zoom_action_group,
 		bacon_video_widget_can_set_zoom (totem->bvw));
+	action = GTK_ACTION (gtk_builder_get_object (totem->xml, "deinterlace"));
+	gtk_action_set_visible (action, bacon_video_widget_can_deinterlace (totem->bvw));
 
 	action = gtk_action_group_get_action (totem->main_action_group, "repeat-mode");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
