@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -30,7 +30,7 @@
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include "plparse/totem-disc.h"
+#include <totem-disc.h>
 #include "totem-playlist.h"
 
 #define TOTEM_GCONF_PREFIX "/apps/totem"
@@ -41,6 +41,7 @@ typedef enum {
 	TOTEM_REMOTE_COMMAND_UNKNOWN,
 	TOTEM_REMOTE_COMMAND_PLAY,
 	TOTEM_REMOTE_COMMAND_PAUSE,
+	TOTEM_REMOTE_COMMAND_STOP,
 	TOTEM_REMOTE_COMMAND_PLAYPAUSE,
 	TOTEM_REMOTE_COMMAND_NEXT,
 	TOTEM_REMOTE_COMMAND_PREVIOUS,
@@ -66,7 +67,8 @@ typedef enum {
 	TOTEM_REMOTE_COMMAND_ZOOM_DOWN,
 	TOTEM_REMOTE_COMMAND_EJECT,
 	TOTEM_REMOTE_COMMAND_PLAY_DVD,
-	TOTEM_REMOTE_COMMAND_MUTE
+	TOTEM_REMOTE_COMMAND_MUTE,
+	TOTEM_REMOTE_COMMAND_TOGGLE_ASPECT
 } TotemRemoteCommand;
 
 typedef enum {
@@ -165,6 +167,7 @@ gboolean totem_is_seekable			(Totem *totem);
 GtkWindow *totem_get_main_window		(Totem *totem);
 GtkUIManager *totem_get_ui_manager		(Totem *totem);
 GtkWidget *totem_get_video_widget		(Totem *totem);
+char *totem_get_video_widget_backend_name	(Totem *totem);
 
 /* Current media information */
 char *	totem_get_short_title			(Totem *totem);
@@ -181,6 +184,7 @@ void totem_add_to_playlist_and_play		(Totem *totem,
 						 const char *uri,
 						 const char *display_name,
 						 gboolean add_to_recent);
+char *  totem_get_current_mrl			(Totem *totem);
 
 /* Sidebar handling */
 void    totem_add_sidebar_page			(Totem *totem,
